#' Read counts of RNA-seq samples of Pasilla knock-down by Brooks et al.
#'
#' A SummarizedExperiment dataset containing 
#' the transcriptome information for Drosophila Melanogaster.
#'
#' @format  containing 14599 features and 7 biological replicates.
#'
#' @source \url{https://bioconductor.org/packages/release/data/experiment/html/pasilla.html}
#' @usage data(pasilla)
"pasilla"

#' Read counts of RNA-seq samples derived from 
#' Pasilla knock-down by Brooks et al.
#'
#' A SummarizedExperiment dataset containing 
#' the transcriptome information for Drosophila Melanogaster.
#'
#' @format  containing 14599 features and 7 biological replicates.
#'
#' @source \url{https://bioconductor.org/packages/release/data/experiment/html/pasilla.html}
#' @usage data(se)
"se"