% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{se}
\alias{se}
\title{Read counts of RNA-seq samples derived from
Pasilla knock-down by Brooks et al.}
\format{
containing 14599 features and 7 biological replicates.
}
\source{
\url{https://bioconductor.org/packages/release/data/experiment/html/pasilla.html}
}
\usage{
data(se)
}
\description{
A SummarizedExperiment dataset containing
the transcriptome information for Drosophila Melanogaster.
}
\keyword{datasets}
