% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{tidy}
\alias{tidy}
\alias{tidy.SummarizedExperiment}
\alias{tidy.RangedSummarizedExperiment}
\title{tidy for \code{Seurat}}
\usage{
tidy(object)

\method{tidy}{SummarizedExperiment}(object)

\method{tidy}{RangedSummarizedExperiment}(object)
}
\arguments{
\item{object}{A SummarizedExperiment object}
}
\value{
A \code{tidyseurat} object.
}
\description{
tidy for \code{Seurat}
}
\examples{
data(pasilla)
pasilla \%>\% tidy()

}
\references{
Hutchison, W.J., Keyes, T.J., The tidyomics Consortium. et al. The tidyomics ecosystem: enhancing omic data analyses. Nat Methods 21, 1166–1170 (2024). https://doi.org/10.1038/s41592-024-02299-2

Hutchison, W.J., Keyes, T.J., The tidyomics Consortium. et al. The tidyomics ecosystem: enhancing omic data analyses. Nat Methods 21, 1166–1170 (2024). https://doi.org/10.1038/s41592-024-02299-2

Hutchison, W.J., Keyes, T.J., The tidyomics Consortium. et al. The tidyomics ecosystem: enhancing omic data analyses. Nat Methods 21, 1166–1170 (2024). https://doi.org/10.1038/s41592-024-02299-2
}
